/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2019-2024 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 *
 **********************************************************************************************************************/
/**
 ***********************************************************************************************************************
 * @file  BuiltIns.h
 * @brief LLPC header file: declaration of BuiltIns supported by the Builder interface
 ***********************************************************************************************************************
 */

#pragma once

namespace lgc {

// Max SPIR-V built-in value
static constexpr unsigned BuiltInInternalBase = 0x10000000;

// Mask to indicate a SPIR-V built-in used in PS semantic (unsigned 16-bit)
static constexpr unsigned BuiltInSemanticMask = 0x8000;

// Define built-in kind enum.
enum BuiltInKind : unsigned {
#define BUILTIN(name, number, out, in, type) BuiltIn##name = number,
#include "lgc/BuiltInDefs.h"
#undef BUILTIN
};

} // namespace lgc
