/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2024-2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 *
 **********************************************************************************************************************/

// NOTE: Assertions have been autogenerated by tool/update_llpc_test_checks.py UTC_ARGS: --function _amdgpu_ps_main
// RUN: amdllpc --print-after=lgc-set-up-target-features %s 2>&1 | FileCheck -check-prefix=SHADERTEST %s

#version 450 core
#define ITER 5
layout(set=0, binding=0) uniform UniformBuffer
{
    vec4 c[4];
    ivec4 ci[4];
    double cd[4];
};

struct TSTR {
    vec4 member;
};

layout(location = 0) out vec4 frag_color;
layout(location = 0) in TSTR interp[ITER];

void main()
{
    int i;
    vec4 s = vec4(0.0f);
    vec2 offset = vec2(0.00001);
    for (i =0; i < ITER; i++)
    {
        offset = vec2(float(i)/float(ITER));
        s += interpolateAtOffset(interp[i].member, offset);
    }
    frag_color.x = float(s.x - interp[0].member.x);
    frag_color.y = float(s.y + interp[0].member.y);
    frag_color.z = 0.0f;
    frag_color.w = 1.0f;
}
// SHADERTEST-LABEL: @_amdgpu_ps_main(
// SHADERTEST-NEXT:  .entry:
// SHADERTEST-NEXT:    [[PERSPINTERPPULLMODE_I2:%.*]] = extractelement <3 x float> [[PERSPINTERPPULLMODE:%.*]], i64 2
// SHADERTEST-NEXT:    [[BC_I2:%.*]] = bitcast float [[PERSPINTERPPULLMODE_I2]] to i32
// SHADERTEST-NEXT:    [[PERSPINTERPPULLMODE_I1:%.*]] = extractelement <3 x float> [[PERSPINTERPPULLMODE]], i64 1
// SHADERTEST-NEXT:    [[BC_I1:%.*]] = bitcast float [[PERSPINTERPPULLMODE_I1]] to i32
// SHADERTEST-NEXT:    [[PERSPINTERPPULLMODE_I0:%.*]] = extractelement <3 x float> [[PERSPINTERPPULLMODE]], i64 0
// SHADERTEST-NEXT:    [[BC_I0:%.*]] = bitcast float [[PERSPINTERPPULLMODE_I0]] to i32
// SHADERTEST-NEXT:    [[PERSPINTERPCENTER_I0:%.*]] = extractelement <2 x float> [[PERSPINTERPCENTER:%.*]], i64 0
// SHADERTEST-NEXT:    [[PERSPINTERPCENTER_I1:%.*]] = extractelement <2 x float> [[PERSPINTERPCENTER]], i64 1
// SHADERTEST-NEXT:    [[TMP0:%.*]] = fdiv reassoc nnan nsz arcp contract afn float 1.000000e+00, [[PERSPINTERPPULLMODE_I2]]
// SHADERTEST-NEXT:    [[DOTI0:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP0]], [[PERSPINTERPPULLMODE_I0]]
// SHADERTEST-NEXT:    [[DOTI1:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP0]], [[PERSPINTERPPULLMODE_I1]]
// SHADERTEST-NEXT:    [[TMP1:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI0]], i32 0, i32 0, i32 [[PRIMMASK:%.*]])
// SHADERTEST-NEXT:    [[TMP2:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP1]], float [[DOTI1]], i32 0, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP3:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI0]], i32 1, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP4:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP3]], float [[DOTI1]], i32 1, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP5:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP6:%.*]] = bitcast i32 [[TMP5]] to float
// SHADERTEST-NEXT:    [[TMP7:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP8:%.*]] = bitcast i32 [[TMP7]] to float
// SHADERTEST-NEXT:    [[TMP9:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP6]], [[TMP8]]
// SHADERTEST-NEXT:    [[TMP10:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP9]])
// SHADERTEST-NEXT:    [[TMP11:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP12:%.*]] = bitcast i32 [[TMP11]] to float
// SHADERTEST-NEXT:    [[TMP13:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP14:%.*]] = bitcast i32 [[TMP13]] to float
// SHADERTEST-NEXT:    [[TMP15:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP12]], [[TMP14]]
// SHADERTEST-NEXT:    [[TMP16:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP15]])
// SHADERTEST-NEXT:    [[TMP17:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP18:%.*]] = bitcast i32 [[TMP17]] to float
// SHADERTEST-NEXT:    [[TMP19:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP20:%.*]] = bitcast i32 [[TMP19]] to float
// SHADERTEST-NEXT:    [[TMP21:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP18]], [[TMP20]]
// SHADERTEST-NEXT:    [[TMP22:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP21]])
// SHADERTEST-NEXT:    [[TMP23:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP24:%.*]] = bitcast i32 [[TMP23]] to float
// SHADERTEST-NEXT:    [[TMP25:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP26:%.*]] = bitcast i32 [[TMP25]] to float
// SHADERTEST-NEXT:    [[TMP27:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP24]], [[TMP26]]
// SHADERTEST-NEXT:    [[TMP28:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP27]])
// SHADERTEST-NEXT:    [[TMP29:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP30:%.*]] = bitcast i32 [[TMP29]] to float
// SHADERTEST-NEXT:    [[TMP31:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP32:%.*]] = bitcast i32 [[TMP31]] to float
// SHADERTEST-NEXT:    [[TMP33:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP30]], [[TMP32]]
// SHADERTEST-NEXT:    [[TMP34:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP33]])
// SHADERTEST-NEXT:    [[TMP35:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP36:%.*]] = bitcast i32 [[TMP35]] to float
// SHADERTEST-NEXT:    [[TMP37:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP38:%.*]] = bitcast i32 [[TMP37]] to float
// SHADERTEST-NEXT:    [[TMP39:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP36]], [[TMP38]]
// SHADERTEST-NEXT:    [[TMP40:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP39]])
// SHADERTEST-NEXT:    [[REASS_ADD_1_I0:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP28]], [[TMP10]]
// SHADERTEST-NEXT:    [[REASS_ADD_1_I1:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP34]], [[TMP16]]
// SHADERTEST-NEXT:    [[REASS_ADD_1_I2:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP40]], [[TMP22]]
// SHADERTEST-NEXT:    [[REASS_MUL_1_I0:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_1_I0]], 0x3FC99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_1_I1:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_1_I1]], 0x3FC99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_1_I2:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_1_I2]], 0x3FC99999A0000000
// SHADERTEST-NEXT:    [[DOTI060:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_1_I0]], [[PERSPINTERPPULLMODE_I0]]
// SHADERTEST-NEXT:    [[DOTI161:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_1_I1]], [[PERSPINTERPPULLMODE_I1]]
// SHADERTEST-NEXT:    [[DOTI2:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_1_I2]], [[PERSPINTERPPULLMODE_I2]]
// SHADERTEST-NEXT:    [[TMP41:%.*]] = fdiv reassoc nnan nsz arcp contract afn float 1.000000e+00, [[DOTI2]]
// SHADERTEST-NEXT:    [[DOTI062:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP41]], [[DOTI060]]
// SHADERTEST-NEXT:    [[DOTI163:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP41]], [[DOTI161]]
// SHADERTEST-NEXT:    [[TMP42:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI062]], i32 0, i32 1, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP43:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP42]], float [[DOTI163]], i32 0, i32 1, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP44:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI062]], i32 1, i32 1, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP45:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP44]], float [[DOTI163]], i32 1, i32 1, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[DOTI064:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP43]], [[TMP2]]
// SHADERTEST-NEXT:    [[DOTI165:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP45]], [[TMP4]]
// SHADERTEST-NEXT:    [[TMP46:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP47:%.*]] = bitcast i32 [[TMP46]] to float
// SHADERTEST-NEXT:    [[TMP48:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP49:%.*]] = bitcast i32 [[TMP48]] to float
// SHADERTEST-NEXT:    [[TMP50:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP47]], [[TMP49]]
// SHADERTEST-NEXT:    [[TMP51:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP50]])
// SHADERTEST-NEXT:    [[TMP52:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP53:%.*]] = bitcast i32 [[TMP52]] to float
// SHADERTEST-NEXT:    [[TMP54:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP55:%.*]] = bitcast i32 [[TMP54]] to float
// SHADERTEST-NEXT:    [[TMP56:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP53]], [[TMP55]]
// SHADERTEST-NEXT:    [[TMP57:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP56]])
// SHADERTEST-NEXT:    [[TMP58:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP59:%.*]] = bitcast i32 [[TMP58]] to float
// SHADERTEST-NEXT:    [[TMP60:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP61:%.*]] = bitcast i32 [[TMP60]] to float
// SHADERTEST-NEXT:    [[TMP62:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP59]], [[TMP61]]
// SHADERTEST-NEXT:    [[TMP63:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP62]])
// SHADERTEST-NEXT:    [[TMP64:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP65:%.*]] = bitcast i32 [[TMP64]] to float
// SHADERTEST-NEXT:    [[TMP66:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP67:%.*]] = bitcast i32 [[TMP66]] to float
// SHADERTEST-NEXT:    [[TMP68:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP65]], [[TMP67]]
// SHADERTEST-NEXT:    [[TMP69:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP68]])
// SHADERTEST-NEXT:    [[TMP70:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP71:%.*]] = bitcast i32 [[TMP70]] to float
// SHADERTEST-NEXT:    [[TMP72:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP73:%.*]] = bitcast i32 [[TMP72]] to float
// SHADERTEST-NEXT:    [[TMP74:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP71]], [[TMP73]]
// SHADERTEST-NEXT:    [[TMP75:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP74]])
// SHADERTEST-NEXT:    [[TMP76:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP77:%.*]] = bitcast i32 [[TMP76]] to float
// SHADERTEST-NEXT:    [[TMP78:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP79:%.*]] = bitcast i32 [[TMP78]] to float
// SHADERTEST-NEXT:    [[TMP80:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP77]], [[TMP79]]
// SHADERTEST-NEXT:    [[TMP81:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP80]])
// SHADERTEST-NEXT:    [[REASS_ADD_2_I0:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP69]], [[TMP51]]
// SHADERTEST-NEXT:    [[REASS_ADD_2_I1:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP75]], [[TMP57]]
// SHADERTEST-NEXT:    [[REASS_ADD_2_I2:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP81]], [[TMP63]]
// SHADERTEST-NEXT:    [[REASS_MUL_2_I0:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_2_I0]], 0x3FD99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_2_I1:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_2_I1]], 0x3FD99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_2_I2:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_2_I2]], 0x3FD99999A0000000
// SHADERTEST-NEXT:    [[DOTI067:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_2_I0]], [[PERSPINTERPPULLMODE_I0]]
// SHADERTEST-NEXT:    [[DOTI168:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_2_I1]], [[PERSPINTERPPULLMODE_I1]]
// SHADERTEST-NEXT:    [[DOTI269:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_2_I2]], [[PERSPINTERPPULLMODE_I2]]
// SHADERTEST-NEXT:    [[TMP82:%.*]] = fdiv reassoc nnan nsz arcp contract afn float 1.000000e+00, [[DOTI269]]
// SHADERTEST-NEXT:    [[DOTI070:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP82]], [[DOTI067]]
// SHADERTEST-NEXT:    [[DOTI171:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP82]], [[DOTI168]]
// SHADERTEST-NEXT:    [[TMP83:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI070]], i32 0, i32 2, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP84:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP83]], float [[DOTI171]], i32 0, i32 2, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP85:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI070]], i32 1, i32 2, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP86:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP85]], float [[DOTI171]], i32 1, i32 2, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[DOTI072:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP84]], [[DOTI064]]
// SHADERTEST-NEXT:    [[DOTI173:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP86]], [[DOTI165]]
// SHADERTEST-NEXT:    [[TMP87:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP88:%.*]] = bitcast i32 [[TMP87]] to float
// SHADERTEST-NEXT:    [[TMP89:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP90:%.*]] = bitcast i32 [[TMP89]] to float
// SHADERTEST-NEXT:    [[TMP91:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP88]], [[TMP90]]
// SHADERTEST-NEXT:    [[TMP92:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP91]])
// SHADERTEST-NEXT:    [[TMP93:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP94:%.*]] = bitcast i32 [[TMP93]] to float
// SHADERTEST-NEXT:    [[TMP95:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP96:%.*]] = bitcast i32 [[TMP95]] to float
// SHADERTEST-NEXT:    [[TMP97:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP94]], [[TMP96]]
// SHADERTEST-NEXT:    [[TMP98:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP97]])
// SHADERTEST-NEXT:    [[TMP99:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP100:%.*]] = bitcast i32 [[TMP99]] to float
// SHADERTEST-NEXT:    [[TMP101:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP102:%.*]] = bitcast i32 [[TMP101]] to float
// SHADERTEST-NEXT:    [[TMP103:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP100]], [[TMP102]]
// SHADERTEST-NEXT:    [[TMP104:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP103]])
// SHADERTEST-NEXT:    [[TMP105:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP106:%.*]] = bitcast i32 [[TMP105]] to float
// SHADERTEST-NEXT:    [[TMP107:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP108:%.*]] = bitcast i32 [[TMP107]] to float
// SHADERTEST-NEXT:    [[TMP109:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP106]], [[TMP108]]
// SHADERTEST-NEXT:    [[TMP110:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP109]])
// SHADERTEST-NEXT:    [[TMP111:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP112:%.*]] = bitcast i32 [[TMP111]] to float
// SHADERTEST-NEXT:    [[TMP113:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP114:%.*]] = bitcast i32 [[TMP113]] to float
// SHADERTEST-NEXT:    [[TMP115:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP112]], [[TMP114]]
// SHADERTEST-NEXT:    [[TMP116:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP115]])
// SHADERTEST-NEXT:    [[TMP117:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP118:%.*]] = bitcast i32 [[TMP117]] to float
// SHADERTEST-NEXT:    [[TMP119:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP120:%.*]] = bitcast i32 [[TMP119]] to float
// SHADERTEST-NEXT:    [[TMP121:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP118]], [[TMP120]]
// SHADERTEST-NEXT:    [[TMP122:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP121]])
// SHADERTEST-NEXT:    [[REASS_ADD_3_I0:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP110]], [[TMP92]]
// SHADERTEST-NEXT:    [[REASS_ADD_3_I1:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP116]], [[TMP98]]
// SHADERTEST-NEXT:    [[REASS_ADD_3_I2:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP122]], [[TMP104]]
// SHADERTEST-NEXT:    [[REASS_MUL_3_I0:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_3_I0]], 0x3FE3333340000000
// SHADERTEST-NEXT:    [[REASS_MUL_3_I1:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_3_I1]], 0x3FE3333340000000
// SHADERTEST-NEXT:    [[REASS_MUL_3_I2:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_3_I2]], 0x3FE3333340000000
// SHADERTEST-NEXT:    [[DOTI076:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_3_I0]], [[PERSPINTERPPULLMODE_I0]]
// SHADERTEST-NEXT:    [[DOTI177:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_3_I1]], [[PERSPINTERPPULLMODE_I1]]
// SHADERTEST-NEXT:    [[DOTI278:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_3_I2]], [[PERSPINTERPPULLMODE_I2]]
// SHADERTEST-NEXT:    [[TMP123:%.*]] = fdiv reassoc nnan nsz arcp contract afn float 1.000000e+00, [[DOTI278]]
// SHADERTEST-NEXT:    [[DOTI079:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP123]], [[DOTI076]]
// SHADERTEST-NEXT:    [[DOTI180:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP123]], [[DOTI177]]
// SHADERTEST-NEXT:    [[TMP124:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI079]], i32 0, i32 3, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP125:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP124]], float [[DOTI180]], i32 0, i32 3, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP126:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI079]], i32 1, i32 3, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP127:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP126]], float [[DOTI180]], i32 1, i32 3, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[DOTI081:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP125]], [[DOTI072]]
// SHADERTEST-NEXT:    [[DOTI182:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP127]], [[DOTI173]]
// SHADERTEST-NEXT:    [[TMP128:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP129:%.*]] = bitcast i32 [[TMP128]] to float
// SHADERTEST-NEXT:    [[TMP130:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP131:%.*]] = bitcast i32 [[TMP130]] to float
// SHADERTEST-NEXT:    [[TMP132:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP129]], [[TMP131]]
// SHADERTEST-NEXT:    [[TMP133:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP132]])
// SHADERTEST-NEXT:    [[TMP134:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP135:%.*]] = bitcast i32 [[TMP134]] to float
// SHADERTEST-NEXT:    [[TMP136:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP137:%.*]] = bitcast i32 [[TMP136]] to float
// SHADERTEST-NEXT:    [[TMP138:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP135]], [[TMP137]]
// SHADERTEST-NEXT:    [[TMP139:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP138]])
// SHADERTEST-NEXT:    [[TMP140:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 245, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP141:%.*]] = bitcast i32 [[TMP140]] to float
// SHADERTEST-NEXT:    [[TMP142:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 160, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP143:%.*]] = bitcast i32 [[TMP142]] to float
// SHADERTEST-NEXT:    [[TMP144:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP141]], [[TMP143]]
// SHADERTEST-NEXT:    [[TMP145:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP144]])
// SHADERTEST-NEXT:    [[TMP146:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP147:%.*]] = bitcast i32 [[TMP146]] to float
// SHADERTEST-NEXT:    [[TMP148:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I0]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP149:%.*]] = bitcast i32 [[TMP148]] to float
// SHADERTEST-NEXT:    [[TMP150:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP147]], [[TMP149]]
// SHADERTEST-NEXT:    [[TMP151:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP150]])
// SHADERTEST-NEXT:    [[TMP152:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP153:%.*]] = bitcast i32 [[TMP152]] to float
// SHADERTEST-NEXT:    [[TMP154:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I1]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP155:%.*]] = bitcast i32 [[TMP154]] to float
// SHADERTEST-NEXT:    [[TMP156:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP153]], [[TMP155]]
// SHADERTEST-NEXT:    [[TMP157:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP156]])
// SHADERTEST-NEXT:    [[TMP158:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 238, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP159:%.*]] = bitcast i32 [[TMP158]] to float
// SHADERTEST-NEXT:    [[TMP160:%.*]] = call i32 @llvm.amdgcn.mov.dpp.i32(i32 [[BC_I2]], i32 68, i32 15, i32 15, i1 true)
// SHADERTEST-NEXT:    [[TMP161:%.*]] = bitcast i32 [[TMP160]] to float
// SHADERTEST-NEXT:    [[TMP162:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[TMP159]], [[TMP161]]
// SHADERTEST-NEXT:    [[TMP163:%.*]] = call reassoc nnan nsz arcp contract afn float @llvm.amdgcn.wqm.f32(float [[TMP162]])
// SHADERTEST-NEXT:    [[REASS_ADD_4_I0:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP151]], [[TMP133]]
// SHADERTEST-NEXT:    [[REASS_ADD_4_I1:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP157]], [[TMP139]]
// SHADERTEST-NEXT:    [[REASS_ADD_4_I2:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP163]], [[TMP145]]
// SHADERTEST-NEXT:    [[REASS_MUL_4_I0:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_4_I0]], 0x3FE99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_4_I1:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_4_I1]], 0x3FE99999A0000000
// SHADERTEST-NEXT:    [[REASS_MUL_4_I2:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[REASS_ADD_4_I2]], 0x3FE99999A0000000
// SHADERTEST-NEXT:    [[DOTI085:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_4_I0]], [[PERSPINTERPPULLMODE_I0]]
// SHADERTEST-NEXT:    [[DOTI186:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_4_I1]], [[PERSPINTERPPULLMODE_I1]]
// SHADERTEST-NEXT:    [[DOTI287:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[REASS_MUL_4_I2]], [[PERSPINTERPPULLMODE_I2]]
// SHADERTEST-NEXT:    [[TMP164:%.*]] = fdiv reassoc nnan nsz arcp contract afn float 1.000000e+00, [[DOTI287]]
// SHADERTEST-NEXT:    [[DOTI088:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP164]], [[DOTI085]]
// SHADERTEST-NEXT:    [[DOTI189:%.*]] = fmul reassoc nnan nsz arcp contract afn float [[TMP164]], [[DOTI186]]
// SHADERTEST-NEXT:    [[TMP165:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI088]], i32 0, i32 4, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP166:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP165]], float [[DOTI189]], i32 0, i32 4, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP167:%.*]] = call float @llvm.amdgcn.interp.p1(float [[DOTI088]], i32 1, i32 4, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP168:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP167]], float [[DOTI189]], i32 1, i32 4, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[DOTI090:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP166]], [[DOTI081]]
// SHADERTEST-NEXT:    [[DOTI191:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[TMP168]], [[DOTI182]]
// SHADERTEST-NEXT:    [[TMP169:%.*]] = call float @llvm.amdgcn.interp.p1(float [[PERSPINTERPCENTER_I0]], i32 0, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP170:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP169]], float [[PERSPINTERPCENTER_I1]], i32 0, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP171:%.*]] = call float @llvm.amdgcn.interp.p1(float [[PERSPINTERPCENTER_I0]], i32 1, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP172:%.*]] = call float @llvm.amdgcn.interp.p2(float [[TMP171]], float [[PERSPINTERPCENTER_I1]], i32 1, i32 0, i32 [[PRIMMASK]])
// SHADERTEST-NEXT:    [[TMP173:%.*]] = fsub reassoc nnan nsz arcp contract afn float [[DOTI090]], [[TMP170]]
// SHADERTEST-NEXT:    [[TMP174:%.*]] = fadd reassoc nnan nsz arcp contract afn float [[DOTI191]], [[TMP172]]
// SHADERTEST-NEXT:    call void @llvm.amdgcn.exp.f32(i32 0, i32 15, float [[TMP173]], float [[TMP174]], float 0.000000e+00, float 1.000000e+00, i1 true, i1 true)
// SHADERTEST-NEXT:    ret void
//
