/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2023-2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 *
 **********************************************************************************************************************/
/**
 ***********************************************************************************************************************
 * @file  GpurtEnums.h
 * @brief Declare enums that are shared between gpurt shader code and the compiler.
 ***********************************************************************************************************************
 */
#pragma once

#ifdef __cplusplus
#include <cstdint>
#endif

/// Shader types.
/// The values of this enum must match DXIL::ShaderKind from DxilConstants.h.
enum class DXILShaderKind : uint32_t {
  Pixel = 0,
  Vertex,
  Geometry,
  Hull,
  Domain,
  Compute,
  Library,
  RayGeneration,
  Intersection,
  AnyHit,
  ClosestHit,
  Miss,
  Callable,
  Mesh,
  Amplification,
  Node,
  Invalid
};

// clang-format off
/// Decimal encoding of rtip levels.
// clang-format on
enum class RayTracingIpLevel : uint32_t {
  _None = 0,
  RtIp1_1 = 11,
  RtIp2_0 = 20,
#if LLPC_BUILD_GFX12
  RtIp3_0 = 30,
#endif
#if LLPC_BUILD_GFX12
  RtIp3_1 = 31,
#endif
};

// CPS Scheduling levels.
enum class CpsSchedulingLevel : uint32_t {
  RayGen = 1,
  ClosestHit_Miss_Callable,
  Traversal,
  AnyHit_CombinedIntersection_AnyHit,
  Intersection,
  Count,
};
